!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              INITIALIZE_TEMPERATURE    Nov. 2006                             |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine initialize_temperature (ov,surface,mat,params)

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! this routine initializes the remperature field to some basic
! conductive equilibrium base on sp01 (baseline) of the surfaces
! temperature should be normalized between 0 and 1
! ov is the velocity octree containing the velocity/temperature solution
! this routine needs to be improved...

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|
      
use definitions

implicit none

include 'mpif.h'

type (octreev) :: ov
type (parameters) :: params
type (sheet) :: surface(params%ns)
type (material) :: mat(0:params%nmat)


!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer iproc, nproc, ierr
integer :: i,j,matnum,nstep,mati
integer :: steptop,stepbottom

double precision :: distance,dist
double precision :: diff,heat,dz,zstep,delz

integer :: matindex(0:500)
double precision :: temp(0:500),tgrad(0:500)

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

select case (params%tempinit)
case (0)
    do i=1,ov%nnode
        ov%temp(i)=max((params%ztemp-ov%z(i))/params%ztemp,0.d0)
    enddo

case (1)
    nstep = 500
    dz = params%ztemp*params%vex/nstep

    !find surface closest above position and set material index to the corresponding 
    !material number

    matindex=0
    matnum=0
    do i=0,nstep
        matnum = 0 
        distance = -2.d0
        do j=1,params%ns
            dist = i*dz-surface(j)%sp01
            if (dist>distance .and. dist<=0.d0) then
                distance = dist
                matnum = surface(j)%material
            end if
        end do
        matindex(i) = matnum
    end do

    temp=0.d0
    tgrad=0.d0

    !first run of temperature init: no temp flux at bottom
    !initialize arrays
    temp(0)=1.d0
    tgrad(0)=0.d0

    do i=1,nstep
        !calculate diffusivity and heat as mean of bottom and top position
        diff = (mat(matindex(i))%diffusivity+mat(matindex(i-1))%diffusivity)*0.5d0
        heat = (mat(matindex(i))%heat+mat(matindex(i-1))%heat)*0.5d0

        tgrad(i) = tgrad(i-1) + heat*dz/diff
        temp(i) = temp(i-1) - (tgrad(i)+tgrad(i-1))*0.5*dz
    end do


    !determine bottom flux as missing temperature
    tgrad(0) = temp(nstep)/(params%ztemp*params%vex)
    temp(0) = 1.d0

    !Redo with bottom heat flux
    do i=1,nstep
        !calculate diffusivity and heat as mean of bottom and top position
        diff = (mat(matindex(i))%diffusivity+mat(matindex(i-1))%diffusivity)*0.5d0
        heat = (mat(matindex(i))%heat+mat(matindex(i-1))%heat)*0.5d0

        tgrad(i) = tgrad(i-1) + heat*dz/diff
        temp(i) = temp(i-1) - (tgrad(i)+tgrad(i-1))*0.5*dz

        !write (*,*) temp(i), tgrad(i)
    end do

    !set temperature by interpolating between closest two points from temperature calculation
    do i=1,ov%nnode
        zstep = ov%z(i)*params%vex/dz
        steptop = ceiling(zstep)
        stepbottom = floor(zstep)
        if (steptop == stepbottom) then
            ov%temp(i) = temp(stepbottom)
        else 
            delz = mod(zstep,1.d0)
            ov%temp(i) = delz*temp(stepbottom)+(1.d0-delz)*temp(steptop)
        end if
        ov%temp(i) = max(0.d0,min(1.d0,ov%temp(i)))
    end do

case default
    call stop_run ('Please choose correct tempinit option')

end select

return
end
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|


